#include "ArmarXCore/core/exceptions/local/ExpressionException.h"
#include <ArmarXCore/core/Component.h>
#include <ArmarXCore/core/application/properties/PropertyDefinitionContainer.h>
#include <ArmarXCore/util/CPPUtility/trace.h>

#include <armarx/navigation/client/ComponentPlugin.h>
#include <armarx/navigation/client/Navigator.h>

// ComponentPlugin

armarx::navigation::client::ComponentPlugin::~ComponentPlugin() = default;

void
armarx::navigation::client::ComponentPlugin::postCreatePropertyDefinitions(
    armarx::PropertyDefinitionsPtr& properties)
{
    if (not properties->hasDefinition(PROPERTY_NAME))
    {
        properties->defineOptionalProperty<std::string>(
            PROPERTY_NAME, "Navigator", "Name of the Navigator");
    }
}

void
armarx::navigation::client::ComponentPlugin::preOnInitComponent()
{
    ARMARX_TRACE;
    parent<armarx::Component>().usingProxyFromProperty(PROPERTY_NAME);
}

void
armarx::navigation::client::ComponentPlugin::preOnConnectComponent()
{
    ARMARX_TRACE;
    parent<armarx::Component>().getProxyFromProperty(navigatorPrx, PROPERTY_NAME);

    ARMARX_TRACE;
    const std::string componentName = parent().getName();

    ARMARX_CHECK_NOT_NULL(navigatorPrx) << "Navigator proxy is null!";
    iceNavigator.setNavigatorComponent(navigatorPrx);
}

void
armarx::navigation::client::ComponentPlugin::configureNavigator(
    const client::NavigationStackConfig& stackConfig,
    const std::string& configId)
{
    ARMARX_TRACE;
    iceNavigator.createConfig(stackConfig, configId);
}

// ComponentPluginUser

armarx::navigation::client::ComponentPluginUser::ComponentPluginUser()
{
    ARMARX_TRACE;
    addPlugin(plugin);
}

void
armarx::navigation::client::ComponentPluginUser::configureNavigator(
    const client::NavigationStackConfig& stackConfig)
{
    ARMARX_TRACE;
    plugin->configureNavigator(stackConfig, getName());
}

armarx::navigation::client::ComponentPluginUser::~ComponentPluginUser() = default;
