/**
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @author     Christian R. G. Dreher ( c dot dreher at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <RobotAPI/components/armem/MemoryNameSystem/MemoryNameSystem.h>

#include <armarx/navigation/memory/client/stack_result/Writer.h>
#include <armarx/navigation/server/introspection/IntrospectorInterface.h>

namespace armarx::navigation::server
{

    class MemoryIntrospector : virtual public IntrospectorInterface
    {
    public:
        MemoryIntrospector(mem::client::stack_result::Writer& globPlanWriter,
                           const std::string& clientID);
        ~MemoryIntrospector() override = default;


        void onGlobalPlannerResult(const glob_plan::GlobalPlannerResult& result) override;
        void onLocalPlannerResult(const loc_plan::LocalPlannerResult& result) override;
        void
        onTrajectoryControllerResult(const traj_ctrl::TrajectoryControllerResult& result) override;
        void onSafetyControllerResult(const safe_ctrl::SafetyControllerResult& result) override;

        void onGoal(const core::Pose& goal) override;

        void
        onGlobalShortestPath(const std::vector<core::Pose>& path) override
        {
        }


    protected:
    private:
        mem::client::stack_result::Writer& globPlanWriter;
        const std::string clientID;
    };
} // namespace armarx::navigation::server
